/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

/* 
 * File:   GLSData.h
 * Author: skidmarks
 *
 * Created on December 15, 2017, 2:30 PM
 */

#ifndef GLSDATA_H
#define GLSDATA_H

# include <sstream>
# include <string>

using namespace std;
namespace gls {

class GLSData {
private:
   GLSData dummy(0, gslDummy, NULL);
public:
   enum gsl { gslDummy = 0
            , gslBlock = 1
            , gslMatrix
            , gslMatrixView
            , gslVector
            , gslVectorView
   }; // gsl
   
protected:
   const void*    structPtr;
   const size_t   totalSize;
   const gsl      ident;
private:
   GLSData(const GLSData& orig) : structPtr(orig.strct()), totalSize(orig.totalSize), ident(orig.ident) {}
   
public:
   GLSData(const size_t size, const gsl ident, const void* ptr) : structPtr(ptr), totalSize(size), ident(ident) {};
   virtual ~GLSData() = 0;
   virtual string toString() = 0;
   
   /******************************************************
    *             Reading and Writing Blocks             *
    ******************************************************/
   
   /******************************************************
    *                Universal  Properties               *
    ******************************************************/
   
   virtual void*  data()     = 0;                        // pointer to gsl_block
   
   inline gsl     id() const    { return ident; };       // block identifier
   inline size_t  size() const  { return totalSize; }    // number of elements in block
   inline void*   strct() const { return const_cast<void*>(structPtr); } // pointer to gsl_block
   
   /******************************************************
    *        Common Matrix and Vector properties         *
    ******************************************************/
   virtual double max()      = 0;               // maximum value
   virtual double min()      = 0;               // maximum value
   
   virtual int    isneg()    = 0;               // true: is negative
   virtual int    isnonneg() = 0;               // true: is not negative
   virtual int    isnull()   = 0;               // true: structure is null
   virtual int    ispos()    = 0;               // true: is positive
      
   /******************************************************
    *    Accessing & initializing Matrices and Vectors   *
    ******************************************************/
   virtual const double * const_ptr(size_t row, size_t col = 0) = 0;
   virtual double * const_ptr(size_t row, size_t col = 0) = 0;
   virtual double get(const size_t row, const size_t col = 0) = 0;
   virtual double* ptr(size_t row, size_t col = 0) = 0;
   virtual void   set_all(double x) = 0;
   virtual void   set_zero() = 0;
   virtual void   set(const double x, const size_t row, const size_t col = 0) = 0;
      
   /******************************************************
    *            Matrix and Vector Arithmetic            *
    ******************************************************/
   virtual int add(GLSData& b) = 0;
   virtual int add_constant(const double x) = 0;
   virtual int sub(GLSData& b) = 0;
   virtual int scale(const double x) = 0;
      
   /******************************************************
    *            Matrix and Vector Operations            *
    ******************************************************/
   virtual int equal(GLSData& b) = 0;
   
   /******************************************************
    *            Matrix and Vector Arithmetic            *
    ******************************************************/
   virtual void   minmax(double * min, double * max)  = 0;
   virtual size_t max_index()  = 0;
   virtual size_t min_index()  = 0;
   virtual void   minmax_index(size_t * imin, size_t * jmin, size_t * imax = 0, size_t* jmax = 0)  = 0;
   
   /******************************************************
    *            Copying Matrices and Vectors            *
    ******************************************************/
   virtual int memcpy(GLSData& dest) = 0;
   virtual int swap(GLSData& dest) = 0;
   
   /******************************************************
    *                  Vector Arithmetic                 *
    ******************************************************/
   virtual int div(GLSData& b) = 0;
   virtual int mul(GLSData& b) = 0;
   
   /******************************************************
    *                        Views                       *
    ******************************************************/
   virtual GLSData view_array(double* base, const size_t number, size_t col=0) = 0;
   virtual const GLSData view_array(const double* base, const size_t number, size_t col=0) = 0;
 
   /******************************************************
    *                    Vector Views                    *
    ******************************************************/
   virtual GLSData subvector(const size_t offset, const size_t number) = 0;
   virtual GLSData subvector_with_stride(const size_t offset, const size_t number, const size_t stride) = 0;
   
   /******************************************************
    *                const Vector Views                  *
    ******************************************************/
   virtual const GLSData subvector(const size_t offset, const size_t number) = 0;
   virtual const GLSData subvector_with_stride(const size_t offset, const size_t number, const size_t stride) = 0;
   
   /******************************************************
    *                    Vector Misc                     *
    ******************************************************/
   virtual int swap_elements(size_t row1, size_t row2) = 0;
   virtual int reverse() = 0;
      
   /******************************************************
    *          Accessing & initializing Matrices         *
    ******************************************************/
   virtual void set_identity() = 0;
  
   /******************************************************
    *                    Matrix Views                    *
    ******************************************************/
   virtual GLSData   submatrix(size_t row, size_t col, size_t rows, size_t cols) = 0;
   virtual GLSData   subrow(size_t row, size_t offset, size_t n) = 0;
   virtual GLSData   view_array_with_tda(double * v, size_t rows, size_t cols, size_t tda) = 0;
   virtual GLSData   view_vector(GLSData * v, size_t rows, size_t cols) = 0;
   virtual GLSData   view_vector_with_tda(GLSData * v, size_t rows, size_t cols, size_t tda) = 0;
  
   /******************************************************
    *                 const Matrix Views                 *
    ******************************************************/
   virtual const GLSData   const_submatrix(size_t row, size_t col, size_t rows, size_t cols) = 0;
   virtual const GLSData   const_view_array_with_tda(double * v, size_t rows, size_t cols, size_t tda) = 0;
   virtual const GLSData   const_view_vector(GLSData * v, size_t rows, size_t cols) = 0;
   virtual const GLSData   const_view_vector_with_tda(GLSData * v, size_t rows, size_t cols, size_t tda) = 0;
  
   /******************************************************
    *            Row and Column Matrix Views             *
    ******************************************************/
   virtual GLSData row(size_t row) = 0;
   virtual GLSData col(size_t col) = 0;
   virtual GLSData subrow(size_t row, size_t offset, size_t number) = 0;
   virtual GLSData subcol(size_t col, size_t offset, size_t number) = 0;
   virtual GLSData diagonal() = 0;
   virtual GLSData subdiagonal(size_t diagonal) = 0;
   virtual GLSData superdiagonal(size_t diagonal) = 0;
  
   /******************************************************
    *        const Row and Column Matrix Views           *
    ******************************************************/
   virtual const GLSData const_row(size_t row) = 0;
   virtual const GLSData const_col(size_t col) = 0;
   virtual const GLSData const_subrow(size_t row, size_t offset, size_t number) = 0;
   virtual const GLSData const_subcol(size_t col, size_t offset, size_t number) = 0;
   virtual const GLSData const_diagonal() = 0;
   virtual const GLSData const_subdiagonal(size_t diagonal) = 0;
   virtual const GLSData const_superdiagonal(size_t diagonal) = 0;
   
   /******************************************************
    *          Matrix Copying Rows and Columns           *
    ******************************************************/
   int get_row(GLSData* v, size_t length) = 0;
   int get_col(GLSData* v, size_t length) = 0;
   int set_row(GLSData* v, size_t row)    = 0;
   int set_col(GLSData* v, size_t col)    = 0;
   
   
   /******************************************************
    *        Matrix Exchanging Rows and Columns          *
    ******************************************************/
   int swap_rows(size_t row1, size_t row2) = 0;
   int swap_cols(size_t col1, size_t col2) = 0;
   int swap_rowcol(size_t row, size_t col) = 0;
   int transpose_memcpy(GLSData* dest) = 0;
   int transpose() = 0;
   
   /******************************************************
    *                  Matrix Operations                 *
    ******************************************************/
   int mul_elements(GLSData* b) = 0;
   int div_elements(GLSData* b) = 0;
      
private:

};
} // namespace gls
#endif /* GLSDATA_H */

